@@ -28,6 +28,11 @@ pub enum Command {
 	#[structopt(name = "key")]
 	Key(KeyOperation),
 
+	/// Run migrations from previous Garage version
+	/// (DO NOT USE WITHOUT READING FULL DOCUMENTATION)
+	#[structopt(name = "migrate")]
+	Migrate(MigrateOpt),
+
 	/// Start repair of node data
 	#[structopt(name = "repair")]
 	Repair(RepairOpt),
@@ -150,6 +155,14 @@ pub enum BucketOperation {
 	#[structopt(name = "delete")]
 	Delete(DeleteBucketOpt),
 
+	/// Alias bucket under new name
+	#[structopt(name = "alias")]
+	Alias(AliasBucketOpt),
+
+	/// Remove bucket alias
+	#[structopt(name = "unalias")]
+	Unalias(UnaliasBucketOpt),
+
 	/// Allow key to read or write to bucket
 	#[structopt(name = "allow")]
 	Allow(PermBucketOpt),
@@ -175,6 +188,14 @@ pub struct WebsiteOpt {
 
 	/// Bucket name
 	pub bucket: String,
+
+	/// Index document: the suffix appended to request paths ending by /
+	#[structopt(short = "i", long = "index-document", default_value = "index.html")]
+	pub index_document: String,
+
+	/// Error document: the optionnal document returned when an error occurs
+	#[structopt(short = "e", long = "error-document")]
+	pub error_document: Option<String>,
 }
 
 #[derive(Serialize, Deserialize, StructOpt, Debug)]
@@ -193,6 +214,29 @@ pub struct DeleteBucketOpt {
 	pub yes: bool,
 }
 
+#[derive(Serialize, Deserialize, StructOpt, Debug)]
+pub struct AliasBucketOpt {
+	/// Existing bucket name (its alias in global namespace or its full hex uuid)
+	pub existing_bucket: String,
+
+	/// New bucket name
+	pub new_name: String,
+
+	/// Make this alias local to the specified access key
+	#[structopt(long = "local")]
+	pub local: Option<String>,
+}
+
+#[derive(Serialize, Deserialize, StructOpt, Debug)]
+pub struct UnaliasBucketOpt {
+	/// Bucket name
+	pub name: String,
+
+	/// Unalias in bucket namespace local to this access key
+	#[structopt(long = "local")]
+	pub local: Option<String>,
+}
+
 #[derive(Serialize, Deserialize, StructOpt, Debug)]
 pub struct PermBucketOpt {
 	/// Access key name or ID
@@ -207,6 +251,11 @@ pub struct PermBucketOpt {
 	#[structopt(long = "write")]
 	pub write: bool,
 
+	/// Allow/deny administrative operations operations
+	/// (such as deleting bucket or changing bucket website configuration)
+	#[structopt(long = "owner")]
+	pub owner: bool,
+
 	/// Bucket name
 	pub bucket: String,
 }
@@ -233,6 +282,14 @@ pub enum KeyOperation {
 	#[structopt(name = "delete")]
 	Delete(KeyDeleteOpt),
 
+	/// Set permission flags for key
+	#[structopt(name = "allow")]
+	Allow(KeyPermOpt),
+
+	/// Unset permission flags for key
+	#[structopt(name = "deny")]
+	Deny(KeyPermOpt),
+
 	/// Import key
 	#[structopt(name = "import")]
 	Import(KeyImportOpt),
@@ -270,6 +327,16 @@ pub struct KeyDeleteOpt {
 	pub yes: bool,
 }
 
+#[derive(Serialize, Deserialize, StructOpt, Debug)]
+pub struct KeyPermOpt {
+	/// ID or name of the key
+	pub key_pattern: String,
+
+	/// Flag that allows key to create buckets using S3's CreateBucket call
+	#[structopt(long = "create-bucket")]
+	pub create_bucket: bool,
+}
+
 #[derive(Serialize, Deserialize, StructOpt, Debug)]
 pub struct KeyImportOpt {
 	/// Access key ID
@@ -283,6 +350,23 @@ pub struct KeyImportOpt {
 	pub name: String,
 }
 
+#[derive(Serialize, Deserialize, StructOpt, Debug, Clone)]
+pub struct MigrateOpt {
+	/// Confirm the launch of the migrate operation
+	#[structopt(long = "yes")]
+	pub yes: bool,
+
+	#[structopt(subcommand)]
+	pub what: MigrateWhat,
+}
+
+#[derive(Serialize, Deserialize, StructOpt, Debug, Eq, PartialEq, Clone)]
+pub enum MigrateWhat {
+	/// Migrate buckets and permissions from v0.5.0
+	#[structopt(name = "buckets050")]
+	Buckets050,
+}
+
 #[derive(Serialize, Deserialize, StructOpt, Debug, Clone)]
 pub struct RepairOpt {
 	/// Launch repair operation on all nodes
